-- *********************************************************
-- **               Custom WoW Audio - Core               **
-- **            http://sweetsour.live/discord            **
-- **          https://www.patreon.com/sweetsour          **
-- *********************************************************
--
-- This addon is written and copyrighted by:
--    • Mike Thornton (Sweetsour @ US-Firetree)
--
--
-- Special thanks to:
--    • /r/wow
--    • My guild, "Star Stack On Star"
--
--
-- The code of this addon is licensed under a Creative Commons Attribution-Noncommercial-Share Alike 3.0 License. (see license.txt)
-- All included textures and sounds are copyrighted by their respective owners, license information for these media files can be found in the modules that make use of them.
--
--
--  You are free:
--    • to Share - to copy, distribute, display, and perform the work
--    • to Remix - to make derivative works
--  Under the following conditions:
--    • Attribution. You must attribute the work in the manner specified by the author or licensor (but not in any way that suggests that they endorse you or your use of the work). (A link to https://www.patreon.com/sweetsour is sufficient)
--    • Noncommercial. You may not use this work for commercial purposes.
--    • Share Alike. If you alter, transform, or build upon this work, you may distribute the resulting work only under the same or similar license to this one.
--

local FOLDER_NAME, Engine = ...

local SCA = {}
SCA.library = {}
SCA.temp = {}
local Ace3 = LibStub('AceAddon-3.0'):NewAddon("CustomAudio", 'AceConsole-3.0','AceEvent-3.0');
Ace3.version = GetAddOnMetadata("CustomAudio", 'Version')

Engine[1] = SCA
Engine[2] = Ace3

------------------------------
--- For Debugging Purposes ---
------------------------------
local BackdropSB = {
	bgFile   = [[Interface\Tooltips\UI-Tooltip-Background]],
	edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
	tile = true,
	tileSize = 16,
	edgeSize = 10
}

local DataFrame = CreateFrame('Frame',nil,UIParent)
DataFrame:SetWidth(700)
DataFrame:SetHeight(300)
DataFrame:SetPoint('TOPLEFT',UIParent,'TOPLEFT',100,-100)
DataFrame:SetBackdrop(BackdropSB)
DataFrame:SetBackdropColor(0.15,0.15,0.15,0.9)
DataFrame:SetBackdropBorderColor(1,1,1,1)
DataFrame:Hide()

DataFrame.text = DataFrame:CreateFontString(nil, 'MEDIUM', 'GameFontHighlightLarge')
DataFrame.text:SetPoint('TOPLEFT',11,-5);
DataFrame.text:SetTextColor(1,1,1,1)
DataFrame.text:SetJustifyH('LEFT')

SCA.DataFrame = DataFrame
_G['SCA_DataFrame'] = DataFrame

local defaults = {
	char = {
		ignore = {
			--[[[51505] = "Lava Burst",
			[188196] = "Lightning Bolt",
			[188443] = "Chain Lightning",]]
		},
		audio = {
			crits = {
				damage = {
					options = {
						pet = true,
						dots = true,
						spells = true,
						threshold = 0,
						source = {
							packs = true,
							custom = false,
						},
						channel = "Master",
					},
					packs = {
						index = 0,
						text = '',
						value = '',
						path = '',
						max = 0,
						fileType = ".mp3",
					},
					custom = {
						path = '',
						max = 0,
						customType = 'file',
						fileType = ".mp3",
					},
				},
				heals = {
					options = {
						pet = true,
						hots = true,
						spells = true,
						threshold = 0,
						source = {
							packs = true,
							custom = false,
						},
						channel = "Master",
					},
					packs = {
						index = 0,
						text = '',
						value = '',
						path = '',
						max = 0,
						fileType = ".mp3",
					},
					custom = {
						path = '',
						max = 0,
						customType = 'file',
						fileType = ".mp3",
					},
				},
			},
		},
	},
}

local function NavigateInterfaceOptions(index)
	local listIndex

	InterfaceOptionsFrame:Show()
	InterfaceOptionsFrameTab2:Click()

	for i=2,InterfaceOptionsFrameAddOns:GetNumChildren() do
		if (select(i,InterfaceOptionsFrameAddOns:GetChildren()):GetText() == "CustomAudio") then
			listIndex = i
			listObject = select(i,InterfaceOptionsFrameAddOns:GetChildren())

			if (select((i+1),InterfaceOptionsFrameAddOns:GetChildren()):GetText() ~= "Crit Sounds") then
				select(1,listObject:GetChildren()):Click()
			end
			break
		end
	end
	
	select(listIndex+index,InterfaceOptionsFrameAddOns:GetChildren()):Click()
end

SLASH_SCA1 = "/sca"
SlashCmdList["SCA"] = function(input)
	if (UnitAffectingCombat('player')) then
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF77Custom WoW Audio|r: |cFFFF7777You cannot access options while in combat.")
	else
		if not input or input:trim() == "" then
			NavigateInterfaceOptions(0)
		elseif input == "crit" then
			NavigateInterfaceOptions(1)
		elseif input == "version" then
			DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF77Custom WoW Audio|r: |cFF9999FF"..GetAddOnMetadata(FOLDER_NAME,"Version").."|r")
		end
	end

end

-- Event: ADDON_LOADED
function Ace3:OnInitialize()
	
	local about_panel = LibStub:GetLibrary("LibAboutPanel", true)

	if about_panel then
		self.optionsFrame = about_panel.new(nil, "CustomAudio")
	end

	self.db = LibStub("AceDB-3.0"):New("SCA_db", defaults)
	_G["SCAdb"] = Ace3.db.char
	self:SetupOptions()
	
	for i = 1, GetNumAddOns() do
		local addonName = GetAddOnInfo(i)
		local enabled = GetAddOnEnableState(UnitName("player"), i)
		if GetAddOnMetadata(i, "X-SCA-SoundPack-GlobalName") and enabled ~= 0 then
			local loaded = LoadAddOn(addonName)
			local voiceGlobal = GetAddOnMetadata(i, "X-SCA-SoundPack-GlobalName")
			local insertFunction = _G[voiceGlobal]
			if loaded and insertFunction then
				insertFunction(SCA)
			else
				print(addonName.." failed to load at time Sound Pack function ran", 2)
			end
		end
	end
end

function Ace3:OnEnable()
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
end

function Ace3:PLAYER_REGEN_DISABLED()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
end

function Ace3:PLAYER_REGEN_ENABLED()
	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
end

local function ThresholdCheck(trigger,inputType,dmgHeal,db)
	if ((db.audio[trigger][inputType].options.threshold > 0 and dmgHeal > db.audio[trigger][inputType].options.threshold) or db.audio[trigger][inputType].options.threshold == 0) then
        return true    
    else
        return false
    end
end

local function CheckValidUnit(unit,db,trigger,inputType)
	if ((unit == UnitName("player") or (unit == UnitName("pet") and db.audio[trigger][inputType].options.pet))) then
		return true
	else
		return false
	end
end

local function CritEventCheck(e,unit,dmgHeal,swingID,db)
	local inputType = ''
	if (((e == "SPELL_PERIODIC_DAMAGE" and db.audio.crits.damage.options.dots) or ((e == "SPELL_DAMAGE" or e == "SWING_DAMAGE")and db.audio.crits.damage.options.spells)) and not db.ignore[swingID]) then
		inputType = 'damage'
    elseif (((e == "SPELL_HEAL" and db.audio.crits.heals.options.spells) or (e == "SPELL_PERIODIC_HEAL" and db.audio.crits.heals.options.hots)) and not db.ignore[swingID]) then
		inputType = 'heals'
    else
        return false
    end

	if (CheckValidUnit(unit,db,'crits',inputType) and ThresholdCheck('crits',inputType,dmgHeal or swingID,db)) then
		--return inputType
		local path,audioSource
			
		if (db.audio.crits[inputType].options.source.packs or (db.audio.crits[inputType].options.source.custom and db.audio.crits[inputType].custom.customType == "pack")) then
			if (db.audio.crits[inputType].options.source.packs) then
				audioSource = "packs"
			else
				audioSource = "custom"
			end
			path = db.audio.crits[inputType][audioSource].path..tostring(math.random(1,db.audio.crits[inputType][audioSource].max or 1))..db.audio.crits[inputType][audioSource].fileType
		else
			path = db.audio.crits[inputType].custom.path..db.audio.crits[inputType].custom.fileType
		end

		PlaySoundFile(path,db.audio.crits[inputType].options.channel)
	else
		--return nil
	end
end

function Ace3:COMBAT_LOG_EVENT_UNFILTERED(event,...)
	local db = Ace3.db.char
	local _,subevent,_,_,unit,_,_,_,_,_,_,swingID,spell,_,dmgHeal,_,_,isHealSwingCrit,_,_,isDmgCrit = ...

	if (isHealSwingCrit or isDmgCrit) then
		CritEventCheck(subevent,unit,dmgHeal,swingID,db)
		--[[if (EventCheck(subevent,unit,dmgHeal,swingID,db)) then
			local path,audioSource
			
			if (db.options.crits.source.packs or (db.options.crits.source.custom and db.audio.custom.customType == "pack")) then
				if (db.options.crits.source.packs) then
					audioSource = "packs"
				else
					audioSource = "custom"
				end
				path = db.audio[audioSource].path..tostring(math.random(1,db.audio[audioSource].max or 1))..db.audio[audioSource].fileType
			else
				path = db.audio.custom.path..db.audio.custom.fileType
			end

			PlaySoundFile(path,db.options.crits.channel)
		end]]
	end
end

_G["SCA_private_db"] = SCA